# -*- coding: utf-8 -*-
from __future__ import absolute_import
from unittest import TestCase
import os

try:
    import pymongo
except ImportError:
    raise ImportError('PyMongo is required for MongoTestCase')


class MongoTestCase(TestCase):
    '''A base for Mongo DB driven test cases. Provides
    :class:`pymongo.MongoClient` instance in :attribute:`mongo_client`
    and has a :method:`purge_database` helper method for database cleanup.

    It is expected that tests are run from `nose` with `--with-mongobox` flag
    that brings up a sandboxed instance of Mongo.
    '''
    __mongo_client = None

    @property
    def mongo_client(self):
        '''Returns an instance of :class:`pymongo.MongoClient` connected
        to MongoBox database instance.
        '''
        if not self.__mongo_client:
            try:
                port = int(os.getenv('MONGOBOX_PORT'))
                self.__mongo_client = pymongo.MongoClient(port=port)
            except (TypeError, pymongo.errors.ConnectionFailure):
                raise RuntimeError(
                    'Seems that MongoBox is not running. ' +
                    'Do you run nosetests with --with-mongobox flag?')

        return self.__mongo_client

    def purge_database(self, drop=True):
        '''Drops all collections in all databases but system ones
        (``system.*``) one by one if :param:`drop` is `True` (default),
        otherwise removes documents using `remove` method.
        Both seem to be faster than dropping databases directly.

        A typical use is call this method in :func:`unittest.TestCase.tearDown`
        to have a clean database for every test case method.

        .. code-block:: python

        def tearDown(self):
            super(self, MyTestCase).tearDown()
            self.purge_database()
        '''

        # exclude system databases
        database_names = (
            db_name for db_name in
            self.mongo_client.database_names()
            if db_name not in ['local']
        )

        for db_name in database_names:
            db = self.mongo_client[db_name]

            # exclude system collections
            collections = (
                db[c] for c in db.collection_names()
                if not c.startswith('system.')
            )
            for collection in collections:
                if drop:
                    db.drop_collection(collection)
                else:
                    try:
                        collection.remove(None)
                    except pymongo.errors.OperationFailure:
                        if collection.options().get('capped', False):
                            # cannot remove documents from capped collections
                            # in latest version of Mongo. Dropping instead.
                            db.drop_collection(collection)



# -*- coding: utf-8 -*-
import os

from nose.plugins import Plugin

from .mongobox import MongoBox


DEFAULT_PORT_ENVVAR = 'MONGOBOX_PORT'


class MongoBoxPlugin(Plugin):
    """A nose plugin that setups a sandboxed mongodb instance.
    """
    name = 'mongobox'

    def options(self, parser, env):
        super(MongoBoxPlugin, self).options(parser, env)
        parser.add_option(
            "--mongobox-bin",
            dest="bin",
            action="store",
            default=None,
            help="Optionally specify the path to the mongod executable.")
        parser.add_option(
            "--mongobox-port",
            action="store",
            dest="port",
            type="int",
            default=0,
            help="Optionally specify the port to run mongodb on.")
        parser.add_option(
            "--mongobox-scripting",
            action="store_true",
            dest="scripting",
            default=False,
            help="Optionally enables mongodb script engine.")
        parser.add_option(
            "--mongobox-dbpath",
            action="store",
            dest="dbpath",
            default=None,
            help=("Path to database files directory. Creates temporary directory by default."))
        parser.add_option(
            "--mongobox-prealloc",
            action="store_true",
            dest="prealloc",
            default=False,
            help=("Optionally preallocate db files"))
        parser.add_option(
            "--mongobox-port-envvar",
            action="store",
            dest="port_envvar",
            default=DEFAULT_PORT_ENVVAR,
            help="Which environment variable dynamic port number will be exported to.")
        parser.add_option(
            "--mongobox-auth",
            action="store",
            dest="auth",
            default=False,
            help="Enable mongodb's user authentication mechanisms.")
        parser.add_option(
            "--mongobox-storage-engine",
            action="store",
            dest="storage_engine",
            default=None,
            help=("Storage engine to use."))

    def configure(self, options, conf):
        super(MongoBoxPlugin, self).configure(options, conf)

        if not self.enabled:
            return

        self.mongobox = MongoBox(
            mongod_bin=options.bin, port=options.port or None,
            db_path=options.dbpath,
            scripting=options.scripting, prealloc=options.prealloc,
            storage_engine=options.storage_engine,
        )

        self.port_envvar = options.port_envvar

    def begin(self):
        assert self.port_envvar not in os.environ, (
            '{} environment variable is already taken. '
            'Do you have other tests with mongobox running?'.format(self.port_envvar))

        self.mongobox.start()
        os.environ[self.port_envvar] = str(self.mongobox.port)

    def finalize(self, result):
        self.mongobox.stop()
        del os.environ[self.port_envvar]



# -*- coding: utf-8 -*-

import os
import tempfile
import subprocess
import time
import sys
import shutil
import socket

from .utils import find_executable, get_free_port

is_windows = lambda: sys.platform.startswith("win")

MONGOD_BIN = 'mongod.exe' if is_windows() else 'mongod'
DEFAULT_ARGS = [
    # don't flood stdout, we're not reading it
    "--quiet",
    # disable unused.
    "--nounixsocket",
    # use a smaller default file size
    "--smallfiles",
    # journaling on by default in 2.0 and makes it to slow
    # for tests, can causes failures in jenkins
    "--nojournal",
]
STARTUP_TIME = 0.4
START_CHECK_ATTEMPTS = 200


class MongoBox(object):
    def __init__(self, mongod_bin=None, port=None,
                 db_path=None, scripting=False,
                 prealloc=False, auth=False, storage_engine=None):

        if db_path and os.path.exists(db_path) and os.path.isfile(db_path):
            raise AssertionError('DB path should be a directory, but it is a file.')

        self.mongod_bin = mongod_bin or find_executable(MONGOD_BIN)

        self.port = port or get_free_port()
        self.scripting = scripting
        self.prealloc = prealloc
        self.db_path = db_path
        self._db_path_is_temporary = not self.db_path
        self.auth = auth
        self.storage_engine = storage_engine

        self.process = None

    def start(self):
        """Start MongoDB.
        """
        if self._db_path_is_temporary:
            self.db_path = tempfile.mkdtemp()
        elif not os.path.exists(self.db_path):
            os.mkdir(self.db_path)

        args = [self.mongod_bin] + list(DEFAULT_ARGS)

        args.extend(['--dbpath', self.db_path])
        args.extend(['--port', str(self.port)])

        self.log_path = os.path.join(self.db_path, 'mongodb.log')
        args.extend(['--logpath', self.log_path])

        if self.storage_engine:
            args.extend(['--storageEngine', self.storage_engine])

        if self.auth:
            args.append("--auth")

        if not self.scripting:
            args.append("--noscripting")

        if not self.prealloc:
            args.append("--noprealloc")

        self.process_args = args

        self.fnull = open(os.devnull, 'w')
        self.process = subprocess.Popen(args, stdout=self.fnull, stderr=subprocess.STDOUT)

        self._wait_till_started()

    def _wait_till_started(self):
        attempts = 0
        while True:
            if self.process.poll() is not None:  # the process has terminated
                with open(self.log_path) as log_file:
                    raise SystemExit('MondgoDB failed to start:\n{}\n{}'.format(
                        ' '.join(self.process_args), log_file.read()))
            attempts += 1
            if attempts > START_CHECK_ATTEMPTS:
                break
            s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
            try:
                try:
                    s.connect(('localhost', int(self.port)))
                    return
                except (IOError, socket.error):
                    time.sleep(0.25)
            finally:
                s.close()

        # MongoDB still does not accept connections. Killing it.
        self.stop()

    def stop(self):
        if self.process is None or self.process.poll() is not None:
            # the process does not exist anymore
            return
        
        # Not sure if there should be more checks for
        # other platforms.
        if sys.platform == 'darwin':
            self.process.kill()
        elif is_windows():
            self.process.terminate()
        else:
            os.kill(self.process.pid, 9)
        self.process.wait()

        if self._db_path_is_temporary:
            shutil.rmtree(self.db_path)
            self.db_path = None

        self.process = None
        self.fnull.close()
        self.fnull = None

    def running(self):
        return self.process is not None

    def client(self):
        import pymongo
        try:
            return pymongo.MongoClient(port=self.port)  # version >=2.4
        except AttributeError:
            return pymongo.Connection(port=self.port)

    def __enter__(self):
        self.start()
        return self

    def __exit__(self, *args, **kwargs):
        self.stop()



# -*- coding: utf-8 -*-
import os
import socket


def find_executable(executable):
    """Scan PATH for an executable.
    """
    for path in os.environ.get('PATH', '').split(os.pathsep):
        path = os.path.abspath(path)
        executable_path = os.path.join(path, executable)
        if os.path.isfile(executable_path):
            return executable_path

    raise AssertionError('Could not find "{}" in system PATH. Make sure you have it installed.'.format(
        executable))


def get_free_port(host="localhost"):
    """Get a free port on the machine.
    """
    temp_sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
    temp_sock.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
    temp_sock.bind((host, 0))
    port = temp_sock.getsockname()[1]
    temp_sock.close()
    del temp_sock
    return port



"""
nose2.cfg should contain:

    [unittest]
    plugins = mongobox.nose2_plugin

    [mongobox]
    # Optionally specify the path to the mongod executable
    # bin =
    # Optionally specify the port to run mongodb on
    # port =
    # Optionally enables mongodb script engine
    # scripting = True
    # Path to database files directory. Creates temporary directory by default
    # dbpath =
    # Optionally store the mongodb log here (default is /dev/null)
    # logpath =
    # Optionally preallocate db files
    # prealloc = True
    # Specify storage engine to use
    # https://docs.mongodb.org/manual/release-notes/3.2-compatibility/#default-storage-engine-change
    # storage_engine =
    # Which environment variable port number will be exported to
    port_envvar = MONGOBOX_PORT

"""
import logging
import os

import nose2.events

from .mongobox import MongoBox


log = logging.getLogger('nose2.plugins.mongobox')


DEFAULT_PORT_ENVVAR = 'MONGOBOX_PORT'


class MongoBoxPlugin(nose2.events.Plugin):
    """A nose plugin that setups a sandboxed mongodb instance.
    """
    configSection = 'mongobox'

    def __init__(self):
        self.register()  # always on
        self.mongobox = MongoBox(
            mongod_bin=self.config.get('bin'),
            port=self.config.as_int('port'),
            db_path=self.config.get('dbpath'),
            scripting=self.config.as_bool('scripting', False),
            prealloc=self.config.as_bool('prealloc', False),
            storage_engine=self.config.get('storage_engine'),
        )
        self.port_envvar = self.config.get('port_envvar', DEFAULT_PORT_ENVVAR)

    def pluginsLoaded(self, event):
        # start MongoDB server here before the tests and the tested application are imported
        # because some apps connect to MongoDB during import
        assert self.port_envvar not in os.environ, (
            '{} environment variable is already taken. '
            'Do you have other tests with mongobox running?'.format(self.port_envvar))

        self.mongobox.start()
        os.environ[self.port_envvar] = str(self.mongobox.port)

    def afterTestRun(self, event):
        self.mongobox.stop()
        del os.environ[self.port_envvar]



from setuptools import setup, find_packages
import mongobox


setup(
    name='mongobox',
    version=mongobox.__version__,
    description="Run sandboxed Mongo DB instance from a python application.",
    long_description=open("README.md").read(),
    url='http://github.com/theorm/mongobox',
    license=open("LICENSE").read(),
    author=mongobox.__author__,
    author_email='roman@kalyakin.com',
    packages=find_packages(exclude=('tests',)),
    package_data={'': ['LICENSE', '*.md']},
    include_package_data=True,
    install_requires=[],
    zip_safe=False,
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Environment :: Plugins',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: Apache Software License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Topic :: Database :: Database Engines/Servers',
        'Topic :: Software Development :: Testing'
    ],
    tests_require=["nose>=0.10", "pymongo>=2.0"],
    test_suite = "nose.collector",
    entry_points={
        'nose.plugins.0.10': [
            'mongobox = mongobox.nose_plugin:MongoBoxPlugin',
        ],
    }
)
